library nui_media;

import 'dart:io';
import 'package:flutter/cupertino.dart';
import 'package:flutter_video_compress/flutter_video_compress.dart';
import 'package:nui_core/code/nui_code_util.dart';
import 'package:nui_core/data/nui_data_util.dart';
import 'package:nui_core/ui/component/nui_bottomsheet_widget.dart';
import 'package:nui_core/ui/component/nui_dialog_widget.dart';
import 'package:nui_core/ui/nui_navigator.dart';
import 'package:nui_media/nui_media_util.dart';
import 'package:nui_media/ui/nui_media_selection_preview.dart';
import 'model/nui_media_file.dart';
export 'package:nui_media/model/nui_media_file.dart';
export 'package:nui_media/model/nui_media_list_repo.dart';
export 'package:nui_media/model/nui_media_models.dart';
export 'package:nui_media/ui/nui_video_list_player.dart';
export 'package:nui_media/ui/nui_video_listview_player.dart';
export 'package:nui_media/ui/nui_video_player.dart';
export 'package:nui_media/ui/nui_media_selection_horizontal.dart';
export 'package:nui_media/ui/nui_media_selection_preview.dart';
export 'package:nui_media/nui_media_util.dart';
export 'package:nui_media/nui_camera_util.dart';

abstract class NUIMediaInterface{
  Future<NUIMediaFile> takePhoto(BuildContext context, {double maxWidth: 1080, double maxHeight: 1080, int imageQuality: 90, bool selfie: false});
  Future<NUIMediaFile> recordVideo(BuildContext context, {double maxWidth: 1080, double maxHeight: 1080, int imageQuality: 90, bool selfie: false});
  Future<List<NUIMediaFile>> getFromGallery({NUIMediaGalleryOption option: NUIMediaGalleryOption.BOTH, int limit:100});
  Future<NUIMediaFile> compressImage(File file, {String targetPath, double maxWidth:1080, double maxHeight: 1080, int quality: 95});
  Future<NUIMediaFile> compressVideoWithBitmap(File file, {String targetPath, int quality, VideoQuality videoQuality: VideoQuality.MediumQuality, bool deleteOrigin: false, bool includeAudio: true, double maxHeight: 1080, double maxWidth: 1080});
  Future<MediaInfo> compressVideo(File file, {String targetPath, VideoQuality quality: VideoQuality.MediumQuality, bool deleteOrigin: false, bool includeAudio: true});
  Future<List<NUIMediaFile>> compressMediaFiles(List<NUIMediaFile> files);
  Future<NUIImageInfo> getImageInfo(String imagePath, {bool local: true});
  Future<NUIVideoInfo> getVideoInfo(String videoPath);
  Future<NUIMediaFile> fileToNUIMediaFile(File file, {bool local: true, MediaType type});
  Future<List<NUIMediaFile>> showFileSelection(BuildContext context, {List<NUIMediaFile> selections, bool enableCrop: true, double cropRatio: 1.0, int minimumSelection, int maximumSelection, NUIMediaGalleryOption type: NUIMediaGalleryOption.BOTH, Color accentColor, Color accentColorBright});
  NUIBottomSheet showMediaSelection(BuildContext context, {String title, TextStyle titleStyle, TextStyle selectionStyle, Color iconColor, Widget galleryIcon, Widget cameraIcon, Widget recordIcon, bool enableGallery: true, bool enableCamera: true, bool enableRecord: true, MediaSelectionListener listener});
}

class NUIMedia implements NUIMediaInterface{
  static const String _DEFAULT_PREFIX = "NUI_";
  static const String _DEFAULT_MODULE = "NUIMediaDefault";
  static Map<String, NUIMedia> _mediaMap = Map();

  String _filenamePrefix;
  String _module;
  NUIMedia._builder(NUIMediaBuilder builder){
    WidgetsFlutterBinding.ensureInitialized();
    _filenamePrefix = builder._filenamePrefix ?? _DEFAULT_PREFIX;
    _module = builder._module ?? _DEFAULT_MODULE;
    _mediaMap[_module] = this;
  }

  static NUIMedia get({String module}){
    NUIMedia instance = _mediaMap[isNullOrEmpty(module) ? _DEFAULT_MODULE : module];
    if(instance == null) throw Exception("NUIMedia -> NUIMedia instance has not been initialized with builder");
    return instance;
  }

  @override
  Future<NUIMediaFile> compressImage(File file, {String targetPath, double maxWidth = 720, double maxHeight = 720, int quality = 90}) {
    return NUIMediaUtil.compressImage(file, targetPath: targetPath, maxWidth: maxWidth, maxHeight: maxHeight, quality: quality);
  }

  @override
  Future<List<NUIMediaFile>> compressMediaFiles(List<NUIMediaFile> files) {
    return NUIMediaUtil.compressMediaFiles(files);
  }

  @override
  Future<MediaInfo> compressVideo(File file, {String targetPath, VideoQuality quality = VideoQuality.MediumQuality, bool deleteOrigin = false, bool includeAudio = true}) {
    return NUIMediaUtil.compressVideo(file, targetPath: targetPath, quality: quality, deleteOrigin: deleteOrigin, includeAudio: includeAudio);
  }

  @override
  Future<NUIMediaFile> compressVideoWithBitmap(File file, {String targetPath, int quality, VideoQuality videoQuality = VideoQuality.MediumQuality, bool deleteOrigin = false, bool includeAudio = true, double maxHeight = 720, double maxWidth = 720}) {
    return NUIMediaUtil.compressVideoWithBitmap(file, targetPath: targetPath, quality: quality, videoQuality: videoQuality, deleteOrigin: deleteOrigin, includeAudio: includeAudio, maxHeight: maxHeight, maxWidth: maxWidth);
  }

  @override
  Future<NUIMediaFile> fileToNUIMediaFile(File file, {bool local = true, MediaType type}) {
    return NUIMediaUtil.fileToMediaFile(file, local: local, type: type);
  }

  @override
  Future<List<NUIMediaFile>> getFromGallery({NUIMediaGalleryOption option = NUIMediaGalleryOption.BOTH, int limit = 100}) {
    return NUIMediaUtil.getAllFromGalleryAsNUIMedias(option: option, limit: limit);
  }

  @override
  Future<NUIImageInfo> getImageInfo(String imagePath, {bool local = true}) {
    return NUIMediaUtil.getImageInfo(imagePath, local: local);
  }

  @override
  Future<NUIVideoInfo> getVideoInfo(String videoPath) {
    return NUIMediaUtil.getVideoInfo(videoPath);
  }

  @override
  Future<NUIMediaFile> recordVideo(BuildContext context, {double maxWidth = 1080, double maxHeight = 1080, int imageQuality = 90, bool selfie = false}) {
    return NUIMediaUtil.recordVideo(context, maxWidth: maxWidth, maxHeight: maxHeight, imageQuality: imageQuality, selfie: selfie);
  }

  @override
  Future<NUIMediaFile> takePhoto(BuildContext context, {double maxWidth = 1080, double maxHeight = 1080, int imageQuality = 90, bool selfie = false}) {
    return NUIMediaUtil.takePhoto(context, maxWidth: maxWidth, maxHeight: maxHeight, imageQuality: imageQuality, selfie: selfie);
  }

  @override
  Future<List<NUIMediaFile>> showFileSelection(BuildContext context, {List<NUIMediaFile> selections, bool enableCrop = true, double cropRatio = 1.0, int minimumSelection, int maximumSelection, NUIMediaGalleryOption type = NUIMediaGalleryOption.BOTH, Color accentColor, Color accentColorBright}) async{
    return await showNUIDialogForResult<List<NUIMediaFile>>(
          context: context,
          content: (context) => NUIMediaSelectionPreview(
            maximumSelection: maximumSelection,
            minimumSelection: minimumSelection,
            type: type,
            accentColor:
            accentColor,
            accentColorBright: accentColorBright,
            selections: selections,
          ),
          opacity: 0.7,
          transition: PageTransition.BLUR_BACKGROUND
      );
  }

  @override
  NUIBottomSheet showMediaSelection(BuildContext context, {String title, TextStyle titleStyle, TextStyle selectionStyle, Color iconColor, Widget galleryIcon, Widget cameraIcon, Widget recordIcon, bool enableGallery = true, bool enableCamera = true, bool enableRecord = true, listener}) {
    return NUIMediaUtil.showMediaSelection(context, title: title, titleStyle: titleStyle, selectionStyle: selectionStyle, iconColor: iconColor, galleryIcon: galleryIcon, cameraIcon: cameraIcon, recordIcon: recordIcon, enableGallery: enableGallery, enableRecord: enableRecord, enableCamera: enableCamera, listener: listener);
  }

}

class NUIMediaBuilder{
  String _filenamePrefix;
  String _module;

  NUIMediaBuilder module(String module){
    this._module = module;
    return this;
  }

  NUIMediaBuilder filenamePrefix(String prefix){
    this._filenamePrefix = prefix;
    return this;
  }

  NUIMedia build(){
    final media = NUIMedia._builder(this);
    logNUI("NUIMedia", "Initialization complete");
    return media;
  }
}