import 'package:better_player/better_player.dart';
import 'package:flutter/cupertino.dart';
import 'package:nui_media/model/nui_media_file.dart';
import 'package:nui_media/nui_media_util.dart';

abstract class NUIVideoPlayerState<T extends StatefulWidget> extends State<T>{
  Widget getVideoPlayer();

  Widget adjustSize({bool adjustSize: false, @required NUIMediaFile mediaFile, double width, double height}) {
    if(adjustSize != true || mediaFile.hasSize() != true || (height == null && width == null)) {
      return getVideoPlayer();
    }
    else{
      final ratio = mediaFile.processHeightToWidthRatio();
      if(width != null && height != null){
        return Container(
          height: height,
          width: width,
          child: getVideoPlayer()
        );
      }
      else if(width != null){
        return Container(
          width: width,
          height: width * ratio,
          child: getVideoPlayer()
        );
      }
      else if(height != null){
        return Container(
          width: height / ratio,
          height: height,
          child: getVideoPlayer()
        );
      }
      return getVideoPlayer();
    }
  }

}

class NUIVideoPlayer extends StatefulWidget {
  NUIVideoPlayer({@required this.mediaFile, this.mute: false, this.boxFit, this.configuration, this.adjustSize: false, this.width, this.height});
  final NUIMediaFile mediaFile;
  final bool adjustSize;
  final double width;
  final double height;
  final bool mute;
  final BoxFit boxFit;
  final BetterPlayerConfiguration configuration;
  @override
  _NUIVideoPlayerState createState() => _NUIVideoPlayerState();
}

class _NUIVideoPlayerState extends NUIVideoPlayerState<NUIVideoPlayer> {
  BetterPlayerController playerController;

  @override
  void initState(){
    super.initState();
    playerController = NUIMediaUtil.getVideoController(widget.mediaFile, config: widget.configuration ?? BetterPlayerConfiguration(fit: widget.boxFit ?? BoxFit.fill));
    playerController.setVolume(widget.mute == true ? 0.0 : 1.0);
  }

  @override
  void dispose(){
    super.dispose();
    playerController?.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return adjustSize(mediaFile: widget.mediaFile, adjustSize: widget.adjustSize, width: widget.width, height: widget.height);
  }

  @override
  Widget getVideoPlayer() => BetterPlayer(controller: playerController);
}
